function ad512(command,p2)
%AD512 Hardware driver setup for the AD512 board.
%
%   This function creates the GUI for setting device driver parameters 
%   for the AD512 board.

%	Version 3.00  14-May-1999
%	Copyright (c) 1992-1999   HUMUSOFT s.r.o.

boardname=rttool('Load','ad512');
figh=findobj('Type','figure','Name',boardname);

if isempty(figh)

% Draw the GUI figure

figh=dialog('HandleVisibility','on','Visible','off','Name',boardname,'Tag','RTDRIVER_Window',...
            'Position',[100 80 285 215],'KeyPressFcn',...
            'ad512 @key');

% Define Address control

uicontrol('Style','text','Position',[20 183 70 20],'HorizontalAlignment','Left',...
          'String','Address :');
uicontrol('Style','edit','BackGroundColor',[1 1 1],'Position',[20 165 40 20],...
          'HorizontalAlignment','Left','CallBack','ad512 @addr;ad512 @cline;',...
          'Tag','Address_Field');
for i=1:7;
  uicontrol('Style','check','Position',[68+27*i 165 20 20],...
            'Max',2^(10-i),'CallBack','ad512 @addrb;ad512 @cline;','Tag','Address_Bits');
  uicontrol('Style','text','Position',[65+27*i 183 24 20],'String',...
            sprintf('A%1d',10-i));
end;

% Define Input format control

uicontrol('Style','text','Position',[30 127 90 20],'HorizontalAlignment','Left',...
          'String','Digital Input:');
uicontrol('Style','radio','Position',[30 110 45 20],'String','Bit','Min',-1,'Max',0,...
          'CallBack','rbutton;ad512 @cline;','Tag','Input_Format');
uicontrol('Style','radio','Position',[80 110 60 20],'String','Byte','Max',2,...
          'CallBack','rbutton;ad512 @cline;','Tag','Input_Format');
  
% Define Output format control

uicontrol('Style','text','Position',[170 127 90 20],'HorizontalAlignment','Left',...
          'String','Digital Output:');
uicontrol('Style','radio','Position',[170 110 45 20],'String','Bit','Min',-1,'Max',0,...
          'CallBack','rbutton;ad512 @cline;','Tag','Output_Format');
uicontrol('Style','radio','Position',[220 110 60 20],'String','Byte','Max',3,...
          'CallBack','rbutton;ad512 @cline;','Tag','Output_Format');

% Define Gain control

uicontrol('Style','push','Position',[190 75 85 25],'String','Gains ...',...
          'CallBack','ad512 @gain','Tag','Gain_Button');

% Define Command line control

uicontrol('Style','text','Position',[10 65 100 16],'HorizontalAlignment','Left',...
          'String','Command line:');
uicontrol('Style','text','BackGroundColor',[1 1 1],'HorizontalAlignment','Left',...
          'Position',[10 45 265 20],'Tag','Command_Line');

% Define buttons

uicontrol('Style','push','Position',[19 10 49 25],'String','OK',...
          'CallBack','ad512 @ok');
uicontrol('Style','push','Position',[85 10 49 25],'String','Reset',...
          'CallBack','ad512 @default');
uicontrol('Style','push','Position',[151 10 49 25],'String','Save',...
          'CallBack','ad512 @cline @save');
uicontrol('Style','push','Position',[217 10 49 25],'String','Cancel',...
          'CallBack','close');
  
% Draw figure

set(findobj(figh,'Units','pixels'),'Units','normalized');
if nargin<2
  drvs=rttool('Read',0,'Drivers');
  drvf=0;
  for i=drvs
    if strcmp(rttool('Read',i,'Name'),'ad512')
      drvf=i;
      break;
    end;
  end;
  if drvf
    [addr,opt]=rttool('Read',drvf,'Address');
    ad512(addr,opt);
  else
    ad512 @default;
  end;
else
  ad512(command,p2);
end;
set(figh,'Visible','on');
drawnow;

else

% Bring the window to front if already open

  if nargin==0
    figure(figh);

% Set default parameters to GUI window

  elseif ~isstr(command)
    set(findobj(figh,'Tag','Address_Field'),'String',dec2hex(command));
    ad512 @addr;
    butt=findobj(figh,'Tag','Input_Format');
    bmax=get(butt,'Max');
    bmin=get(butt,'Min');
    bmax=[bmax{:}]-[bmin{:}];
    rbutton(butt(find(bmax==rem(p2(1),2)+1)));
    butt=findobj(figh,'Tag','Output_Format');
    bmax=get(butt,'Max');
    bmin=get(butt,'Min');
    bmax=[bmax{:}]-[bmin{:}];
    rbutton(butt(find(bmax==rem(p2(1),4)-rem(p2(1),2)+1)));
    gbv=zeros(4,8);
    gains=[p2(2:size(p2,2)) zeros(1,9-size(p2,2))];
    for i=1:8
      gbv(find([0:3]==gains(i)),i)=1;
    end;
    set(findobj(figh,'Tag','Gain_Button'),'UserData',gbv);
    ad512 @cline;

% Call if Address bits changed

  elseif strcmp(command,'@addrb')
    addr = get(findobj(figh,'Tag','Address_Bits'), 'Value');
    addr = sum([addr{:} 0]);
    set(findobj(figh,'Tag','Address_Field'),'Value',addr,'String',dec2hex(addr));
   
% Call if Address field changed

  elseif strcmp(command,'@addr')
    a=hex2dec(get(findobj(figh,'Tag','Address_Field'),'String'));
    addrbh=findobj(figh,'Tag','Address_Bits');
    for i=addrbh'
      bmax=get(i,'Max');
      set(i,'Value',bmax*rem(fix(a/bmax),2));
    end;
    ad512 @addrb;

% Call if Gain pressed

  elseif strcmp(command,'@gain')
    gbv=get(findobj(figh,'Tag','Gain_Button'),'UserData');
    gbh=zeros(size(gbv));
    chfh=dialog('Visible','off',...
                'Name','Select Input Gains',...
                'Position',[102 138 280 98]);
    for i=1:8
      uicontrol('Style','text','Position',[70+15*i 81 9 12],...
                'String',rem(i,10));
      for j=1:4
        gbh(j,i)=uicontrol('Style','radio','Position',[67+15*i 80-18*j 15 15],...
                           'Value',gbv(j,i),'CallBack','rbutton',...
                           'Tag',sprintf('Gain_%d',i));
      end;
    end;

    gains=str2mat('0-5 V','5 V','0-10 V','10 V');
    for j=1:4
      uicontrol('Style','push','Position',[7 79-18*j 70 18],...
                'String',[gains(j,:) ' >>'],...
                'CallBack',sprintf('ad512(''@gainbutt'',%d)',j));
    end;
    uicontrol('Style','push','Position',[220 45 50 25],...
              'String','OK',...
              'CallBack','ad512 @gainok');
    uicontrol('Style','push','Position',[220 10 50 25],...
              'String','Cancel',...
              'CallBack','close');

    set(chfh,'UserData',gbh,'Visible','on');

% Call if Gain OK pressed

  elseif strcmp(command,'@gainok')
    gbh=get(gcf,'UserData');
    gbv=get(gbh,'Value');
    set(findobj(figh,'Tag','Gain_Button'), 'UserData', reshape([gbv{:}],size(gbh)));
    close;
    ad512 @cline;

% Call if Gain button pressed

  elseif strcmp(command,'@gainbutt')
    gbh=get(gcf,'UserData');
    for i=gbh(p2,:)
      rbutton(i);
    end;

% Call if Command line to be updated

  elseif strcmp(command,'@cline')
    addr=get(findobj(figh,'Tag','Address_Field'),'Value');
    ioform = get([findobj(figh,'Tag','Input_Format')' findobj(figh,'Tag','Output_Format')'], 'Value');
    ioform = sum([ioform{:}]);
    og=[0:3]*get(findobj(figh,'Tag','Gain_Button'),'UserData');
    cl=sprintf('rtload(''ad512'',%d,[%d',addr,ioform);
    cl=[cl sprintf(' %d',og)];
    cl=[cl ']);'];
    set(findobj(figh,'Tag','Command_Line'),'String',cl);
    if nargin==2
      opt=str2num(cl(min(find(cl=='[')):max(find(cl==']'))));
      [f p]=uiputfile('ad512.mat','Save driver defaults');
      if f
        save([p f],'addr','opt');
      end;
    end;

% Call if OK pressed

  elseif strcmp(command,'@ok')
    try
      rteval_(get(findobj(figh,'Tag','Command_Line'),'String'));
      close;
    catch
      rterror(lasterr);
    end;

% Call if Reset pressed

  elseif strcmp(command,'@default')
    if exist('ad512.mat')==2
      load ad512;
    else
      [dummy,addr,opt]=rttool('Load','ad512');
    end;
    ad512(addr,opt);

% Call if key pressed

  elseif strcmp(command,'@key')
    ch=get(figh,'CurrentCharacter');
    if ch==27
      close;
    elseif ch==13
      ad512 @ok;
    elseif upper(ch)=='R'
      ad512 @revert;
    elseif upper(ch)=='S'
      ad512 @cline @save;
    elseif upper(ch)=='G'
      ad512 @gain;
    end;

  end

end


